***			AEJMacro2013-0231_Feenstra_SectorEstimates									***
***																						***
***	This program:																		***
***		Estimates sector-level substitution elasticities using Feenstra (1994)			***
***		When Feenstra (1994) does not work, perform a grid search algorithm following 	***
***		Broda and Weinstein (2006)
***		Aggregate sector-level elasticities to comptue the aggregate trade elasticity	***
***	Input: data.temp_bil.sas7bdat (product-level estimation database)					***
***	       data.weight_IO_MO_wkva.sas7bdat (weight matrix)		                		***
***	       prog.panel_sitc_unc_nohetero_hs6FE.sas (estimation without the           	***
***		heteroscedasticity correction)													***
***	       prog.panel_sitc_unc_hetero_FE.sas (estimation with the		        		***
***		heteroscedasticity correction)													***
***	       prog.GridSearch.sas (Grid Search algorithm)			        				***
***	Output: res.sigma.sas7bdat (product-level estimations)								***
***	        res.sol_u.sas7bdat (Unconstrained trade elasticity)							***	;


goptions devmap=winansi keymap=winansi;
options pagesize=100 linesize=100;
OPTIONS OBS=10000 NOREPLACE; *SAS won't process any data, not replace any existing data;
OPTIONS OBS=MAX REPLACE; *SAS will proceed;


libname data "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra";* where the raw data are saved;
libname res "W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra\Output";*where the output is saved;


%let prog= W:\Elast_JI\Output\AEJ\AdditionalMaterial\Feenstra;* where the programs are saved;
%let sect= isic3;*sectoral level ISIC rev3; 

/**********************************************************************************************/

* import data;
data temp_bil;
	set data.temp_bil;
data weight;
	set data.weight_IO_MO_wkva;
run;

* first-stage estimation;
proc sort data=temp_bil;
	by hs6;
proc glm noprint data=temp_bil;
	by hs6;
	class i;
	model dls2 dlps =i /noint p  nouni;
	output out=base_unc p=s2hat pshat;
run;quit;

* estimation without heteroscedasticity correction + residual computation;
%include "&prog\panel_sitc_unc_nohetero_hs6FE.sas";

* instrument residuals using country-specific dummies (no sectorial dimension);
proc glm noprint data=base_unc;
	class i;
	model res2=i/p nouni;
	output out=base_unc p=r2hat;
run;quit;

* Correct for heteroscedasticity;
data base_unc;
	set base_unc;
	rhat=sqrt(r2hat);
	Y=dlp2/rhat;
	X0=1/rhat;
	X1=s2hat/rhat;
	X2=pshat/rhat;
run;

* unconstrained estimation with heteroscedasticity correction;
proc sort data=base_unc;
	by &sect;
%let basesol=work.sol_unc;
%let basevar=work.var_cov_unc;
%let basermse=work.rmse_unc;
%include "&prog\panel_sitc_unc_hetero_FE.sas";/*panel by sitc, with hs6 fixed effects*/

* Compute sectoral rho, omega and sigma from theta1 and theta2;
data sol_unc;
	set sol_unc;
	hs6=substr(Parameter,11,6);
data FE_unc;
	set sol_unc;
	where hs6^=' ';
data FE_unc;
	set FE_unc;
	keep &sect hs6 estimate;
run;
data X1;
	set sol_unc;
	where parameter="X1";
	rename estimate=X1;
	VARX1=StdErr*StdErr;
data X2;
	set sol_unc;
	where parameter="X2";
	rename estimate=X2;
	VARX2=StdErr*StdErr;
data COVX1X2;
	set var_cov_unc;
	where parameter='X1';
	COVX1X2=X2;
data COVX1X2;
	set COVX1X2;
	keep &sect COVX1X2;
proc sort data=COVX1X2;
	by &sect;
proc sort data=X1;
	by &sect;
proc sort data=X2;
	by &sect;
data sigma;
	merge X1(keep=&sect X1 VARX1) X2(keep=&sect X2 VARX2) COVX1X2;
 	by &sect;
data sigma;
	set sigma;
	tX1=abs(X1/(VARX1**0.5));
	if tX1<1.67 then X1=0;
	tX2=abs(X2/(VARX2**0.5));
	if tX2<1.67 then X2=0;
	rac=sqrt(X2*X2+4*X1);
	if X1>0 then epsilon=(X2+rac)/(-2*X1);
	omega=-epsilon*X1;
	domdep=-X1;
	domdX1=-epsilon;
	depdX1=(X2*X2+2*X1+X2*rac)/(2*X1*X1*rac);
	depdX2=(rac+X2)/(-2*X1*rac);
	COVepsilonX1=depdX1*VARX1+depdX2*COVX1X2;
	VARepsilon=(depdX1*depdX1)*VARX1+2*depdX1*depdX2*COVX1X2+(depdX2*depdX2)*VARX2;
	VARomega=(domdep*domdep)*VARepsilon+2*domdep*domdX1*COVepsilonX1+(domdX1*domdX1)*VARX1;
data sigma;
	set sigma;
	where X1^=.;
run;
proc univariate data=sigma;
	var epsilon;
run;quit;

* Grid search in those sectors where X1 is not significant;
data sol_GS;
	set sigma;
	where epsilon=.;
data sectGS;
	set sol_GS(keep=&sect );
	nsec=_n_;
proc sort data=temp_bil;
	by &sect;
proc sort data=sectGS;
	by &sect;
data base_GS;
	merge temp_bil sectGS(keep=&sect nsec );
	by &sect;
data base_GS;
	set base_GS;
	where nsec^=.;
run;

%include "&prog\GridSearch.sas";
data sigma;
	set sigma;
	where epsilon^=.;
data sigma;
	set sigma(keep=&sect epsilon omega VARepsilon VARomega) sol_gs(keep=&sect epsilon omega VARepsilon) ;
run;
data res.sigma;
	set sigma;
	if VARomega=. then VARomega=0;
run;
proc sort data=res.sigma;
	by isic3;
run;

* Aggregate substitution elasticities;
proc sort data=sigma;
	by &sect;
proc sort data=weight;
	by &sect;
data sigma;
	merge sigma weight;
	by &sect;
	wkmwk=wk*wKM;
	drop nk;
proc univariate noprint data=sigma;
	where epsilon^=.;
	var wkmwk ;
	output out=sumwkmwk sum=sumwkmwk ;
run;
data sumwkmwk;
	set sumwkmwk;
	weight=1;
data sigma;
	set sigma;
	weight=1;
proc sort data=sumwkmwk;
	by weight;
proc sort data=sigma;
	by weight;
data sigma;
	merge sigma sumwkmwk;
	by weight;
data sigma;
	set sigma;
	nk=wKM*wk/sumwkmwk;
proc univariate noprint data=sigma;
	where epsilon^=.;
	var nk;
	output out=sumnk sum=sumnk;
run;
data sumnk;
	set sumnk;
	weight=1;
proc sort data=sumnk;
	by weight;
proc sort data=sigma;
	by weight;
data sigma;
	merge sigma sumnk;
	by weight;
	nk=nk/sumnk;
	nksigmak=nk*(1-epsilon);
	nksigmawkm=nk*(-epsilon)*wKM;
	var_elimpt_tot=(nk*(1-wKM))**2*VARepsilon;
proc univariate noprint data=sigma;
	where nksigmak^=.;
	var nksigmak nksigmawkm var_elimpt_tot;
	output out=sol_u sum=nksigmak nksigmawkm var_elimpt_tot;
run;
data sol_u;
	set sol_u;
	elimpt_tot=1-nksigmak+nksigmawkm;
	se_elimpt_tot=sqrt(var_elimpt_tot);
run;
data res.sol_u;
	set sol_u;
	keep elimpt_tot  se_elimpt_tot;
run;

